"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const localize_1 = __importDefault(require("../../i18n/localize"));
const AI_1 = __importDefault(require("../../tictactoe/ai/AI"));
class GameBoardBuilder {
    constructor() {
        this.emojies = ['⬜', '🇽', '🅾️'];
        this.title = '';
        this.stateKey = '';
        this.boardSize = 0;
        this.boardData = [];
    }
    withTitle(player1, player2) {
        this.title =
            localize_1.default.__('game.title', {
                player1: player1.displayName,
                player2: player2.displayName
            }) + '\n\n';
        return this;
    }
    withEmojies(first, second, none) {
        this.emojies = [none !== null && none !== void 0 ? none : this.emojies[0], first, second];
        return this;
    }
    withBoard(boardSize, board) {
        this.boardSize = boardSize;
        this.boardData = board;
        return this;
    }
    withLoadingMessage() {
        this.stateKey = 'game.load';
        return this;
    }
    withEntityPlaying(entity, emojiIndex) {
        this.stateEntity = { name: entity.toString(), emojiIndex: emojiIndex };
        this.stateKey = entity instanceof AI_1.default ? 'game.waiting-ai' : 'game.action';
        return this;
    }
    withEndingMessage(winner) {
        if (winner) {
            this.stateKey = 'game.win';
            this.stateEntity = { name: winner.toString() };
        }
        else {
            this.stateKey = 'game.end';
        }
        return this;
    }
    withExpireMessage() {
        this.stateKey = 'game.expire';
        return this;
    }
    withEmbed(embedColor) {
        this.embedColor = embedColor;
        return this;
    }
    toMessageOptions() {
        let board = '';
        for (let i = 0; i < this.boardSize * this.boardSize; i++) {
            board += this.emojies[this.boardData[i]] + ' ';
            if ((i + 1) % this.boardSize === 0) {
                board += '\n';
            }
        }
        const state = this.generateState();
        const stateWithBoard = `${board}${board && state ? '\n' : ''}${state}`;
        return {
            allowedMentions: { parse: ['users'] },
            embeds: this.embedColor
                ? [{ title: this.title, description: stateWithBoard, color: this.embedColor }]
                : [],
            content: !this.embedColor ? this.title + stateWithBoard : undefined,
            components: []
        };
    }
    generateState() {
        var _a, _b;
        let player = (_a = this.stateEntity) === null || _a === void 0 ? void 0 : _a.name;
        if (((_b = this.stateEntity) === null || _b === void 0 ? void 0 : _b.emojiIndex) !== undefined) {
            player += ` ${this.emojies[this.stateEntity.emojiIndex]}`;
        }
        return localize_1.default.__(this.stateKey, player ? { player } : undefined);
    }
}
exports.default = GameBoardBuilder;
GameBoardBuilder.MOVE_REACTIONS = ['↖️', '⬆️', '↗️', '⬅️', '⏺️', '➡️', '↙️', '⬇️', '↘️'];
